﻿using System;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using ThinkGeo.MapSuite.Core;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.IO;
using System.Text;
using System.Collections.ObjectModel;

public partial class _Default : System.Web.UI.Page, ICallbackEventHandler {
    private const double bufferFactor = 10;
    private string callbackResult = String.Empty;

    protected void Page_Load(object sender, EventArgs e) {
        if (!Page.IsPostBack) {
            Map1.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.FromArgb(255, 233, 232, 214));
            Map1.CurrentExtent = new RectangleShape(-131.22, 55.05, -54.03, 16.91);
            Map1.MapUnit = GeographyUnit.DecimalDegree;

            ShapeFileFeatureLayer stateLayer = new ShapeFileFeatureLayer(Server.MapPath(@"App_Data\STATES.SHP"));
            stateLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.LocalRoad1;
            stateLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            stateLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.MajorStreet2("FENAME");
            stateLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            stateLayer.DrawingMarginPercentage = 100;

            Map1.StaticOverlay.Layers.Add("RoadLayer", stateLayer);
        }
    }

    #region ICallbackEventHandler Members

    public string GetCallbackResult() {
        return callbackResult;
    }

    public void RaiseCallbackEvent(string eventArgument) {
        ShapeFileFeatureLayer roadLayer = (ShapeFileFeatureLayer)Map1.StaticOverlay.Layers["RoadLayer"];

        DataContractJsonSerializer serializer = new DataContractJsonSerializer(typeof(ClickInformation));
        ClickInformation clickInfo = (ClickInformation)serializer.ReadObject(new MemoryStream(Encoding.Default.GetBytes(eventArgument)));
        RectangleShape extent = clickInfo.ReadExtent();
        Map1.CurrentExtent = extent;
        PointShape clickScreenPoint = clickInfo.ReadPointShape();
        PointShape clickWorldPoint = Map1.ToWorldCoordinate((float)clickScreenPoint.X, (float)clickScreenPoint.Y);

        roadLayer.Open();
        Collection<Feature> features = roadLayer.QueryTools.GetFeaturesNearestTo(new Feature(clickWorldPoint), Map1.MapUnit, 1, new string[] { "STATE_NAME" });
        roadLayer.Close();

        if (features.Count != 0) {
            Feature feature = features[0];
            string feName = feature.ColumnValues["STATE_NAME"];
            callbackResult = feName;
        }
        else {
            callbackResult = String.Empty;
        }
    }
    #endregion
}

[DataContract]
public class ClickInformation {
    [DataMember(Name = "x")]
    public double X { get; set; }

    [DataMember(Name = "y")]
    public double Y { get; set; }

    [DataMember(Name = "l")]
    public double Left { get; set; }

    [DataMember(Name = "t")]
    public double Top { get; set; }

    [DataMember(Name = "r")]
    public double Right { get; set; }

    [DataMember(Name = "b")]
    public double Bottom { get; set; }

    public PointShape ReadPointShape() {
        return new PointShape(X, Y);
    }

    public RectangleShape ReadExtent() {
        return new RectangleShape(Left, Top, Right, Bottom);
    }
}
